/**
 * Proyecto: 9x04-Bicola - Vector y Ficheros
 * Fichero:  Bicola.java
 * Fecha:    24/05/2012
 *
 * Licencia (CC) BY-NC-SA: giltesa.com
 *
 * http://docs.oracle.com/javase/7/docs/api/java/util/Vector.html
 * http://mundogeek.net/archivos/2009/03/11/generics-en-java/
 */


import java.util.Vector;
/*import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;*/
import java.io.*;


public class Bicola<G> implements Serializable
{

	// PROPIEDADES:
	private Vector<Nodo<G>>	  vNodos;
	private static final long	serialVersionUID	= 3717013638745390665L;


	// CONSTRUCTOR:
	// Se instancia un Vector de Nodos Genericos. (Del tipo con el que instanciamos la Bicola).
	public Bicola()
	{
		vNodos = new Vector<Nodo<G>>(5, 2);
	}


	// METODOS:
	// Se devuelve el numero de elementos usados en el Vector.
	public int getNumDeNodos()
	{
		return vNodos.size();
	}


	// Se aade un Nodo a la izquierda y se desplazan los demas una posicion a la derecha en el indice.
	public void addNodoIzquierda( G dato )
	{
		vNodos.add(0, new Nodo<G>(dato));
	}


	// Se aade al final del Vector un nuevo Nodo:	
	public void addNodoDerecha( G dato )
	{
		vNodos.addElement(new Nodo<G>(dato));
	}


	// Se copia el primer elemento en temp, despues se elimina del Vector.
	public G getNodoIzquierda()
	{
		G temp = vNodos.firstElement().getDatoNodo();
		vNodos.removeElementAt(0);
		return temp;
	}


	// Se copia el ultimo elemento en temp, despues se elimina del Vector.	
	public G getNodoDerecha()
	{
		G temp = vNodos.lastElement().getDatoNodo();
		vNodos.removeElementAt(vNodos.size() - 1);
		return temp;
	}


	// Se concatenan todos los elementos del Vector en cadena y luego se devuelve.
	public String toString()
	{
		String cadena = "";

		for( int i = 0; i < vNodos.size(); i++ )
			cadena += vNodos.get(i).getDatoNodo() + ", ";

		return cadena;
	}


	// Lee un objecto Bicola desde el fichero backup y se devuelve por "valor".
	public Bicola<G> readBicolaFichero()
	{
		FileInputStream fichEntrada = null; // Revisar: Esto sobra?
		ObjectInputStream objEntrada = null;
		Bicola<G> temp = null;


		try
		{
			fichEntrada = new FileInputStream("backup.dat");
			objEntrada = new ObjectInputStream(fichEntrada);
			temp = (Bicola<G>) objEntrada.readObject();
		}
		catch( FileNotFoundException e )
		{
			System.out.println("El sistema no puede encontrar el archivo especificado");
		}
		catch( ClassNotFoundException e )
		{
			System.out.println("El contenido leido en \"backup.dat\" no era el esperado.");
		}
		catch( Exception e )
		{
			System.out.println(e);
		}
		finally
		{
			// Como ultimo paso se cierran los ficheros.
			try
			{
				if( objEntrada != null )
					objEntrada.close();
				if( fichEntrada != null )
					fichEntrada.close();
			}
			catch( Exception e )
			{
				System.out.println(e);
			}

		}

		return temp;
	}

	
	// Recibe por referencia un objeto Bicola que es guardado en el fichero backup.
	public void writeBicolaFichero( Bicola<G> bicola )
	{
		FileOutputStream fichSalida = null;
		ObjectOutputStream objSalida = null;

		try
		{
			fichSalida = new FileOutputStream("backup.dat");
			objSalida = new ObjectOutputStream(fichSalida);
			objSalida.writeObject(bicola);
		}
		catch( FileNotFoundException e )
		{
			System.out.println("El sistema no puede encontrar el archivo especificado");
		}
		catch( Exception e )
		{
			System.out.println(e);
		}
		finally
		{
			// Como ultimo paso se cierran los ficheros.
			try
			{
				if( objSalida != null )
					objSalida.close();
				if( fichSalida != null )
					fichSalida.close();
			}
			catch( Exception e )
			{
				System.out.println("No se pudo cerrar el fichero.");
			}

		}
	}
}
